<?php
App::uses('EvShopAppModel', 'EvShop.Model');
/**
 * Product Model
 *
 * @property Brand $Brand
 * @property TaxLevel $TaxLevel
 * @property Variant $Variant
 * @property Category $Category
 */
class Product extends EvShopAppModel {

	public $actsAs = array(
		'Routable.Routable' => array(
			'config' => 'EvShop'
		),
		'MetaData.Meta',
		'EvTemplates.Template' => array(
			'formInject' => true
		)
	);

	/**
	 * Display field
	 *
	 * @var string
	 */
	public $displayField = 'name';

	/**
	 * Validation rules
	 *
	 * @var array
	 */
	public $validate = array(
		'name' => array(
			'notEmpty' => array(
				'rule' => array('notBlank'),
				'message' => 'A product name must be entered'
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

	/**
	 * belongsTo associations
	 *
	 * @var array
	 */
	public $belongsTo = array(
		'Brand' => array(
			'className' => 'EvShop.Brand'
		)
	);

	/**
	 * hasMany associations
	 *
	 * @var array
	 */
	public $hasMany = array(
		'Variant' => array(
			'className' => 'EvShop.Variant'
		)
	);

	/**
	 * hasAndBelongsToMany associations
	 *
	 * @var array
	 */
	public $hasAndBelongsToMany = array(
		'Category' => array(
			'className' => 'EvShop.Category',
			'joinTable' => 'ev_shop_categories_products',
			'unique' => 'keepExisting'
		)
	);

	/**
	 * image slots
	 */
	public $imageSlots = array(
		'listing' => array(
			'slots' => 1,
			'fields' => false
		)
	);

	/**
	 * redefine constructor to check for EvTax
	 */
	public function __construct($id = false, $table = null, $ds = null) {
		if (CakePlugin::loaded('EvTax')) {
			$this->hasMany['TaxLevel'] = array(
				'className' => 'EvTax.TaxLevel'
			);
		}

		parent::__construct($id, $table, $ds);
	}

	/**
	 * before we save anything process the options into variants
	 *
	 */
	public function beforeBeforeSave($data, $options) {
		$Variant = EvClassRegistry::init('EvShop.Variant');

		if (empty($data['CurrentOptions'])) {
			$data['CurrentOptions'] = array();
		}
		if (empty($data['Options'])) {
			$data['Options'] = array();
		}

		$this->variantsToDelete = $Variant->processDeletedOptions($data['CurrentOptions'], $data['Options'], $data[$this->alias][$this->primaryKey]);
		$data['Variant'] = $Variant->processNewOptions($data['CurrentOptions'], $data['Options'], $data[$this->alias]['name']);
		unset($data['Options']);

		return $data;
	}

	/**
	 * readForEdit - bring out the variants and calculate the options
	 *
	 */
	public function readForEdit($id, $query = array()) {
		$query['contain']['Variant'] = array('Option');
		$query['contain'][] = 'Category';

		$data = parent::readForEdit($id, $query);
		$data['Options'] = Hash::combine($data['Variant'], '{n}.Option.{n}.id', '{n}.Option.{n}.id', '{n}.Option.{n}.option_group_id');

		return $data;
	}
}
